@@ -31,10 +31,6 @@ class Clinic(Analysis):
                  sp_tracker_track_memory=True,
                  optimization_passes=None,
                  ):
-
-        # Delayed import
-        import ailment.analyses  # pylint:disable=redefined-outer-name,unused-import,import-outside-toplevel
-
         if not func.normalized:
             raise ValueError("Decompilation must work on normalized function graphs.")
 
@@ -119,7 +115,7 @@ class Clinic(Analysis):
         ail_graph = self._simplify_blocks(stack_pointer_tracker=spt)
 
         # Simplify the entire function for the first time
-        self._simplify_function(ail_graph)
+        self._simplify_function(ail_graph, unify_variables=False)
 
         # clear _blocks_by_addr_and_size so no one can use it again
         # TODO: Totally remove this dict
@@ -129,7 +125,9 @@ class Clinic(Analysis):
         self._make_callsites(ail_graph, stack_pointer_tracker=spt)
 
         # Simplify the entire function for the second time
-        self._simplify_function(ail_graph)
+        # TODO: Simplify until reaching a fixed point instead of calling _simplify_function() twice.
+        self._simplify_function(ail_graph, unify_variables=True)
+        self._simplify_function(ail_graph, unify_variables=True)
 
         # Make function arguments
         arg_list = self._make_argument_list()
@@ -239,7 +237,7 @@ class Clinic(Analysis):
         return simp.result_block
 
     @timethis
-    def _simplify_function(self, ail_graph):
+    def _simplify_function(self, ail_graph, unify_variables=False):
         """
         Simplify the entire function.
 
@@ -254,7 +252,8 @@ class Clinic(Analysis):
             self.function,
             func_graph=ail_graph,
             remove_dead_memdefs=self._remove_dead_memdefs,
-            reaching_definitions=rd
+            reaching_definitions=rd,
+            unify_variables=unify_variables,
         )
 
         def _handler(node):
@@ -388,11 +387,12 @@ class Clinic(Analysis):
         vr = self.project.analyses.VariableRecoveryFast(self.function,  # pylint:disable=unused-variable
                                                         func_graph=ail_graph, kb=tmp_kb, track_sp=False,
                                                         func_args=arg_list)
-        # clean up existing types
+        # clean up existing types for this function
         tmp_kb.variables[self.function.addr].remove_types()
+        # TODO: Type inference for global variables
         # run type inference
         try:
-            tp = self.project.analyses.Typehoon(vr.type_constraints, kb=tmp_kb)
+            tp = self.project.analyses.Typehoon(vr.type_constraints, kb=tmp_kb, var_mapping=vr.var_to_typevar)
             tp.update_variable_types(self.function.addr, vr.var_to_typevar)
         except Exception:  # pylint:disable=broad-except
             l.warning("Typehoon analysis failed. Variables will not have types. Please report to GitHub.",
@@ -447,13 +447,16 @@ class Clinic(Analysis):
                 self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt, stmt.condition)
 
             elif stmt_type is ailment.Stmt.Call:
-                if stmt.args:
-                    for arg in stmt.args:
-                        self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt,
-                                                     arg)
-                if stmt.ret_expr:
-                    self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt,
-                                                 stmt.ret_expr)
+                self._link_variables_on_call(variable_manager, global_variables, block, stmt_idx, stmt, is_expr=False)
+
+    def _link_variables_on_call(self, variable_manager, global_variables, block, stmt_idx, stmt, is_expr=False):
+        if stmt.args:
+            for arg in stmt.args:
+                self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt,
+                                             arg)
+        if not is_expr and stmt.ret_expr:
+            self._link_variables_on_expr(variable_manager, global_variables, block, stmt_idx, stmt,
+                                         stmt.ret_expr)
 
     def _link_variables_on_expr(self, variable_manager, global_variables, block, stmt_idx, stmt, expr):
         """
@@ -527,6 +530,9 @@ class Clinic(Analysis):
                 expr.variable = var
                 expr.variable_offset = 0
 
+        elif isinstance(expr, ailment.Stmt.Call):
+            self._link_variables_on_call(variable_manager, global_variables, block, stmt_idx, expr, is_expr=True)
+
     def _function_graph_to_ail_graph(self, func_graph):
 
         node_to_block_mapping = {}
