@@ -6,17 +6,19 @@ import pyvex
 from ...engines.light import SimEngineLight, SimEngineLightVEXMixin, SpOffset, RegisterOffset
 from ...engines.vex.claripy.irop import operations as vex_operations
 from ...errors import SimEngineError
-from ...calling_conventions import DEFAULT_CC, SimRegArg, SimStackArg
+from ...calling_conventions import DEFAULT_CC, SimRegArg, SimStackArg, SimCC
+from ...utils.constants import DEFAULT_STATEMENT
 from ...knowledge_plugins.key_definitions.definition import Definition
-from ...knowledge_plugins.key_definitions.atoms import Register, MemoryLocation, Parameter, Tmp
+from ...knowledge_plugins.key_definitions.tag import LocalVariableTag, ParameterTag, ReturnValueTag, Tag
+from ...knowledge_plugins.key_definitions.atoms import Atom, Register, MemoryLocation, Parameter, Tmp
 from ...knowledge_plugins.key_definitions.constants import OP_BEFORE, OP_AFTER
 from ...knowledge_plugins.key_definitions.dataset import DataSet
-from ...knowledge_plugins.key_definitions.undefined import Undefined, undefined
+from ...knowledge_plugins.key_definitions.undefined import Undefined, UNDEFINED
+from ...code_location import CodeLocation
 from .rd_state import ReachingDefinitionsState
 from .external_codeloc import ExternalCodeLocation
 
 if TYPE_CHECKING:
-    from ...calling_conventions import SimCC
     from ...knowledge_plugins import FunctionManager
 
 
@@ -27,13 +29,13 @@ class SimEngineRDVEX(
     SimEngineLightVEXMixin,
     SimEngineLight,
 ):  # pylint:disable=abstract-method
-    def __init__(self, project, current_local_call_depth, maximum_local_call_depth, functions=None,
+    def __init__(self, project, call_stack, maximum_local_call_depth, functions=None,
                  function_handler=None):
         super(SimEngineRDVEX, self).__init__()
         self.project = project
-        self.functions: Optional['FunctionManager'] = functions
-        self._current_local_call_depth = current_local_call_depth
+        self._call_stack = call_stack
         self._maximum_local_call_depth = maximum_local_call_depth
+        self.functions: Optional['FunctionManager'] = functions
         self._function_handler = function_handler
         self._visited_blocks = None
         self._dep_graph = None
@@ -56,7 +58,22 @@ class SimEngineRDVEX(
             if kwargs.pop('fail_fast', False) is True:
                 raise e
             l.error(e)
-        return self.state, self._visited_blocks
+        return self.state, self._visited_blocks, self._dep_graph
+
+    def _process_block_end(self):
+        self.stmt_idx = DEFAULT_STATEMENT
+        if self.block.vex.jumpkind == "Ijk_Call":
+            # it has to be a function
+            addr = self._expr(self.block.vex.next)
+            self._handle_function(addr)
+        elif self.block.vex.jumpkind == "Ijk_Boring":
+            # test if the target addr is a function or not
+            addr = self._expr(self.block.vex.next)
+            if len(addr) == 1:
+                addr_int = next(iter(addr.data))
+                if isinstance(addr_int, int) and addr_int in self.functions:
+                    # yes it's a jump to a function
+                    self._handle_function(addr)
 
     #
     # Private methods
@@ -150,10 +167,25 @@ class SimEngineRDVEX(
                     l.info('Data to write at address %#x undefined, ins_addr = %#x.', a, self.ins_addr)
 
                 if isinstance(a, int) or (isinstance(a, SpOffset) and isinstance(a.offset, int)):
+                    tags: Optional[Set[Tag]]
+                    if isinstance(a, SpOffset):
+                        function_address = (
+                            self.project.kb
+                                .cfgs['CFGFast']
+                                .get_all_nodes(self._codeloc().ins_addr, anyaddr=True)[0]
+                                .function_address
+                        )
+                        tags = {LocalVariableTag(
+                            function = function_address,
+                            metadata = {'tagged_by': 'SimEngineRDVEX._store_core'}
+                        )}
+                    else:
+                        tags = None
+
                     memloc = MemoryLocation(a, size)
                     # different addresses are not killed by a subsequent iteration, because kill only removes entries
                     # with same index and same size
-                    self.state.kill_and_add_definition(memloc, self._codeloc(), data)
+                    self.state.kill_and_add_definition(memloc, self._codeloc(), data, tags=tags)
 
     def _handle_LoadG(self, stmt):
         guard: DataSet = self._expr(stmt.guard)
@@ -217,7 +249,7 @@ class SimEngineRDVEX(
         data = super()._expr(expr)
         if data is None:
             bits = expr.result_size(self.tyenv)
-            data = DataSet(undefined, bits)
+            data = DataSet(UNDEFINED, bits)
         return data
 
     def _handle_RdTmp(self, expr: pyvex.IRExpr.RdTmp) -> Optional[DataSet]:
@@ -243,7 +275,7 @@ class SimEngineRDVEX(
             data.update(current_def.data)
         if len(data) == 0:
             # no defs can be found. add a fake definition
-            data.add(undefined)
+            data.add(UNDEFINED)
             self.state.kill_and_add_definition(Register(reg_offset, size), self._external_codeloc(),
                                                DataSet(data, bits))
         if any(type(d) is Undefined for d in data):
@@ -293,14 +325,14 @@ class SimEngineRDVEX(
                     for def_ in current_defs:
                         data.update(def_.data)
                 else:
-                    data.add(undefined)
+                    data.add(UNDEFINED)
                 memory_location = MemoryLocation(a, size)
                 self.state.add_use(memory_location, self._codeloc())
             else:
                 l.info('Memory address undefined, ins_addr = %#x.', self.ins_addr)
 
         if len(data) == 0:
-            data.add(undefined)
+            data.add(UNDEFINED)
 
         return DataSet(data, size * self.arch.byte_width)
 
@@ -397,7 +429,7 @@ class SimEngineRDVEX(
                         head = ((1 << e1) - 1) << (bits - e1)
                     data.add(head | (e0 >> e1))
                 except (ValueError, TypeError) as e:
-                    data.add(undefined)
+                    data.add(UNDEFINED)
                     l.warning(e)
 
         return DataSet(data, expr.result_size(self.tyenv))
@@ -468,7 +500,7 @@ class SimEngineRDVEX(
         bits = expr.result_size(self.tyenv)
         for arg_expr in expr.args:
             self._expr(arg_expr)
-        return DataSet(undefined, bits)
+        return DataSet(UNDEFINED, bits)
 
     #
     # User defined high level statement handlers
@@ -481,7 +513,7 @@ class SimEngineRDVEX(
 
     def _handle_function_core(self, func_addr: Optional[DataSet], **kwargs) -> bool:  # pylint:disable=unused-argument
 
-        if self._current_local_call_depth > self._maximum_local_call_depth:
+        if len(self._call_stack) + 1 > self._maximum_local_call_depth:
             l.warning('The analysis reached its maximum recursion depth.')
             return False
 
@@ -489,7 +521,7 @@ class SimEngineRDVEX(
             l.warning('Invalid type %s for IP.', type(func_addr).__name__)
             handler_name = 'handle_unknown_call'
             if hasattr(self._function_handler, handler_name):
-                executed_rda, state = getattr(self._function_handler, handler_name)(self.state, self._codeloc())
+                executed_rda, state = getattr(self._function_handler, handler_name)(self.state, src_codeloc=self._codeloc())
                 state: ReachingDefinitionsState
                 self.state = state
             else:
@@ -500,7 +532,7 @@ class SimEngineRDVEX(
             # indirect call
             handler_name = 'handle_indirect_call'
             if hasattr(self._function_handler, handler_name):
-                _, state = getattr(self._function_handler, handler_name)(self.state, self._codeloc())
+                _, state = getattr(self._function_handler, handler_name)(self.state, src_codeloc=self._codeloc())
                 self.state = state
             else:
                 l.warning('Please implement the indirect function handler with your own logic.')
@@ -511,7 +543,8 @@ class SimEngineRDVEX(
             l.warning('Invalid type %s for IP.', type(func_addr_int).__name__)
             handler_name = 'handle_unknown_call'
             if hasattr(self._function_handler, handler_name):
-                executed_rda, state = getattr(self._function_handler, handler_name)(self.state, self._codeloc())
+                executed_rda, state = getattr(self._function_handler, handler_name)(self.state,
+                                                                                    src_codeloc=self._codeloc())
                 state: ReachingDefinitionsState
                 self.state = state
             else:
@@ -520,19 +553,20 @@ class SimEngineRDVEX(
 
         # direct calls
         ext_func_name = None
-        if self.project.loader.main_object.contains_addr(func_addr_int):
-            ext_func_name = self.project.loader.find_plt_stub_name(func_addr_int)
-        else:
+        if not self.project.loader.main_object.contains_addr(func_addr_int):
+            is_internal = False
             symbol = self.project.loader.find_symbol(func_addr_int)
             if symbol is not None:
                 ext_func_name = symbol.name
-        is_internal = ext_func_name is None
+        else:
+            is_internal = True
 
         executed_rda = False
         if ext_func_name is not None:
             handler_name = 'handle_%s' % ext_func_name
             if hasattr(self._function_handler, handler_name):
-                executed_rda, state = getattr(self._function_handler, handler_name)(self.state, self._codeloc())
+                codeloc = CodeLocation(func_addr_int, 0, None, func_addr_int, context=self._context)
+                executed_rda, state = getattr(self._function_handler, handler_name)(self.state, codeloc)
                 self.state = state
             else:
                 l.warning('Please implement the external function handler for %s() with your own logic.',
@@ -544,14 +578,22 @@ class SimEngineRDVEX(
         elif is_internal is True:
             handler_name = 'handle_local_function'
             if hasattr(self._function_handler, handler_name):
-                executed_rda, state = getattr(self._function_handler, handler_name)(
+                codeloc = CodeLocation(func_addr_int, 0, None, func_addr_int, context=self._context)
+                executed_rda, state, visited_blocks, dep_graph = getattr(self._function_handler, handler_name)(
                     self.state,
                     func_addr_int,
-                    self._current_local_call_depth + 1,
+                    self._call_stack,
                     self._maximum_local_call_depth,
-                    self._codeloc(),
+                    self._visited_blocks,
+                    self._dep_graph,
+                    src_ins_addr=self.ins_addr,
+                    codeloc=codeloc,
                 )
-                self.state = state
+                if executed_rda:
+                    # update everything
+                    self.state = state
+                    self._visited_blocks = visited_blocks
+                    self._dep_graph = dep_graph
             else:
                 # l.warning('Please implement the local function handler with your own logic.')
                 pass
@@ -559,7 +601,8 @@ class SimEngineRDVEX(
             l.warning('Could not find function name for external function at address %#x.', func_addr_int)
             handler_name = 'handle_unknown_call'
             if hasattr(self._function_handler, handler_name):
-                executed_rda, state = getattr(self._function_handler, handler_name)(self.state, self._codeloc())
+                executed_rda, state = getattr(self._function_handler, handler_name)(self.state,
+                                                                                    src_codeloc=self._codeloc())
                 self.state = state
             else:
                 l.warning('Please implement the unknown function handler with your own logic.')
@@ -567,43 +610,47 @@ class SimEngineRDVEX(
 
         return skip_cc
 
-    def _handle_function_cc(self, func_addr: Optional[DataSet]):  # pylint:disable=unused-argument
-
-        cc: Optional['SimCC'] = None
+    def _handle_function_cc(self, func_addr: Optional[DataSet]):
+        _cc = None
+        func_addr_int: Optional[Union[int,Undefined]] = None
         if func_addr is not None and len(func_addr) == 1 and self.functions is not None:
             func_addr_int = next(iter(func_addr))
             if self.functions.contains_addr(func_addr_int):
-                cc = self.functions[func_addr_int].calling_convention
+                _cc = self.functions[func_addr_int].calling_convention
+        cc: SimCC = _cc or DEFAULT_CC.get(self.arch.name, None)(self.arch)
 
-        if cc is None:
-            cc = DEFAULT_CC.get(self.arch.name, None)(self.arch)
+        # follow the calling convention and:
+        # - add uses for arguments
+        # - kill return value registers
+        # - caller-saving registers
+        if cc.args:
+            code_loc = self._codeloc()
+            for arg in cc.args:
+                if isinstance(arg, SimRegArg):
+                    reg_offset, reg_size = self.arch.registers[arg.reg_name]
+                    atom = Register(reg_offset, reg_size)
+                elif isinstance(arg, SimStackArg):
+                    atom = MemoryLocation(SpOffset(self.arch.bits,
+                                          arg.stack_offset),
+                                          arg.size * self.arch.byte_width)
+                self.state.add_use(atom, code_loc)
+                self._tag_definitions_of_atom(atom, func_addr_int)
 
-        if cc is not None:
-            # follow the calling convention and:
-            # - add uses for arguments
-            # - kill return value registers
-            # - caller-saving registers
-            if cc.args:
-                code_loc = self._codeloc()
-                for arg in cc.args:
-                    if isinstance(arg, SimRegArg):
-                        reg_offset, reg_size = self.arch.registers[arg.reg_name]
-                        self.state.add_use(Register(reg_offset, reg_size), code_loc)
-                    elif isinstance(arg, SimStackArg):
-                        self.state.add_use(MemoryLocation(SpOffset(self.arch.bits,
-                                                                   arg.stack_offset),
-                                                          arg.size * self.arch.byte_width),
-                                           code_loc)
-            if cc.RETURN_VAL is not None:
-                if isinstance(cc.RETURN_VAL, SimRegArg):
-                    reg_offset, reg_size = self.arch.registers[cc.RETURN_VAL.reg_name]
-                    atom = Register(reg_offset, reg_size)
-                    self.state.kill_and_add_definition(atom, self._codeloc(), DataSet({undefined}, reg_size * 8))
-            if cc.CALLER_SAVED_REGS is not None:
-                for reg in cc.CALLER_SAVED_REGS:
-                    reg_offset, reg_size = self.arch.registers[reg]
-                    atom = Register(reg_offset, reg_size)
-                    self.state.kill_and_add_definition(atom, self._codeloc(), DataSet({undefined}, reg_size * 8))
+        if cc.RETURN_VAL is not None:
+            if isinstance(cc.RETURN_VAL, SimRegArg):
+                reg_offset, reg_size = self.arch.registers[cc.RETURN_VAL.reg_name]
+                atom = Register(reg_offset, reg_size)
+                tag = ReturnValueTag(
+                    function = func_addr_int if isinstance(func_addr_int, int) else None,
+                    metadata = {'tagged_by': 'SimEngineRDVEX._handle_function_cc'}
+                )
+                self.state.kill_and_add_definition(atom, self._codeloc(), DataSet({UNDEFINED}, reg_size * 8), tags={tag})
+
+        if cc.CALLER_SAVED_REGS is not None:
+            for reg in cc.CALLER_SAVED_REGS:
+                reg_offset, reg_size = self.arch.registers[reg]
+                atom = Register(reg_offset, reg_size)
+                self.state.kill_and_add_definition(atom, self._codeloc(), DataSet({UNDEFINED}, reg_size * 8))
 
         if self.arch.call_pushes_ret is True:
             # pop return address if necessary
@@ -618,7 +665,8 @@ class SimEngineRDVEX(
                     sp_data.update(d.data)
 
             if len(sp_data) != 1:
-                raise ValueError('Invalid number of values for stack pointer.')
+                l.critical('Invalid number of values for stack pointer. Stack is probably unbalanced. This indicates '
+                           'serious problems with function handlers. Stack pointer values include: %s.', sp_data)
 
             sp_addr = next(iter(sp_data))
             if isinstance(sp_addr, (int, SpOffset)):
@@ -629,4 +677,17 @@ class SimEngineRDVEX(
                 raise TypeError('Invalid type %s for stack pointer.' % type(sp_addr).__name__)
 
             atom = Register(self.arch.sp_offset, self.arch.bytes)
-            self.state.kill_and_add_definition(atom, self._codeloc(), DataSet(sp_addr, self.arch.bits))
+            tag = ReturnValueTag(
+                function = func_addr_int,
+                metadata = {'tagged_by': 'SimEngineRDVEX._handle_function_cc'}
+            )
+            self.state.kill_and_add_definition(atom, self._codeloc(), DataSet(sp_addr, self.arch.bits), tags={tag})
+
+    def _tag_definitions_of_atom(self, atom: Atom, func_addr: int):
+        definitions = self.state.get_definitions(atom)
+        tag = ParameterTag(
+            function = func_addr,
+            metadata = {'tagged_by': 'SimEngineRDVEX._handle_function_cc'}
+        )
+        for definition in definitions:
+            definition.tags |= {tag}
