@@ -346,7 +346,7 @@ class Function(Serializable):
                 l.warning("The new calling convention for %r does not have a prototype associated. Using the existing "
                           "function prototype to update the new calling convention. The existing function prototype "
                           "will be removed.", self)
-                self._cc.func_ty = self._prototype
+                self._cc.set_func_type_with_arch(self._prototype)
                 self._prototype = None
             elif self._cc.func_ty is not None and self._prototype is not None:
                 l.warning("The new calling convention for %r already has a prototype associated. The existing function "
@@ -376,7 +376,7 @@ class Function(Serializable):
         :return:    None
         """
         if self._cc:
-            self._cc.func_ty = proto.with_arch(self.project.arch) if proto else None
+            self._cc.set_func_type_with_arch(proto)
         else:
             self._prototype = proto
 
@@ -534,24 +534,6 @@ class Function(Serializable):
 
         return constants
 
-    @property
-    def runtime_values(self):
-        """
-        All of the concrete values used by this function at runtime (i.e., including passed-in arguments and global
-        values).
-        """
-        constants = set()
-        for b in self.block_addrs:
-            for sirsb in self._function_manager._cfg.get_all_irsbs(b):
-                for s in sirsb.successors + sirsb.unsat_successors:
-                    for a in s.history.recent_actions:
-                        for ao in a.all_objects:
-                            if not isinstance(ao.ast, claripy.ast.Base):
-                                constants.add(ao.ast)
-                            elif not ao.ast.symbolic:
-                                constants.add(s.solver.eval(ao.ast))
-        return constants
-
     @property
     def num_arguments(self):
         return len(self._argument_registers) + len(self._argument_stack_variables)
@@ -1434,7 +1416,7 @@ class Function(Serializable):
         self.prototype = proto
         if self.calling_convention is not None:
             self.calling_convention.args = None
-            self.calling_convention.func_ty = proto
+            self.calling_convention.set_func_type_with_arch(proto)
 
     @staticmethod
     def _addr_to_funcloc(addr):
@@ -1491,13 +1473,14 @@ class Function(Serializable):
         """
         :return: The set of all functions that can be reached from the function represented by self.
         """
-        def _called(function_address) -> Set[int]:
-            successors = set(self._function_manager.callgraph.successors(function_address))
-            called_functions = successors.copy()
+        called = set()
+        def _find_called(function_address):
+            successors = set(self._function_manager.callgraph.successors(function_address)) - called
             for s in successors:
-                called_functions |= _called(s)
-            return called_functions
-        return { self._function_manager.function(a) for a in _called(self.addr) }
+                called.add(s)
+                _find_called(s)
+        _find_called(self.addr)
+        return { self._function_manager.function(a) for a in called }
 
     def copy(self):
         func = Function(self._function_manager, self.addr, name=self.name, syscall=self.is_syscall)
