@@ -1,9 +1,10 @@
+# pylint:disable=abstract-method
 from collections import OrderedDict, defaultdict
 from .misc.ux import deprecated
 import copy
 import re
 import logging
-from typing import Optional
+from typing import Optional, Dict, Any, Tuple, List
 
 import claripy
 
@@ -18,6 +19,11 @@ try:
 except ImportError:
     pycparser = None
 
+try:
+    import CppHeaderParser
+except ImportError:
+    CppHeaderParser = None
+
 
 class SimType:
     """
@@ -63,6 +69,15 @@ class SimType:
     def name(self):
         return repr(self)
 
+    @name.setter
+    def name(self, v):
+        raise NotImplementedError("Please implement name setter for %s or consider subclassing NamedTypeMixin."
+                                  % self.__class__)
+
+    def unqualified_name(self, lang: str="c++") -> str:
+        raise NotImplementedError("Please implement unqualified_name for %s or consider subclassing NamedTypeMixin."
+                                  % self.__class__)
+
     def _refine_dir(self): # pylint: disable=no-self-use
         return []
 
@@ -106,6 +121,38 @@ class SimType:
     def c_repr(self):
         raise NotImplementedError()
 
+    def copy(self):
+        raise NotImplementedError()
+
+
+class NamedTypeMixin:
+    """
+    SimType classes with this mixin in the class hierarchy allows setting custom class names. A typical use case is
+    to represent same or similar type classes with different qualified names, such as "std::basic_string" vs
+    "std::__cxx11::basic_string". In such cases, .name stores the qualified name, and .unqualified_name() returns the
+    unqualified name of the type.
+    """
+    def __init__(self, *args, name: Optional[str]=None, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._name = name
+
+    @property
+    def name(self) -> str:
+        if self._name is None:
+            self._name = repr(self)
+        return self._name
+
+    @name.setter
+    def name(self, v):
+        self._name = v
+
+    def unqualified_name(self, lang: str="c++") -> str:
+        if lang == "c++":
+            splitter = "::"
+            n = self.name.split(splitter)
+            return n[-1]
+        raise NotImplementedError("Unsupported language %s." % lang)
+
 
 class SimTypeBottom(SimType):
     """
@@ -126,6 +173,9 @@ class SimTypeBottom(SimType):
             return self.label
         return "BOT"
 
+    def copy(self):
+        return SimTypeBottom(self.label)
+
 
 class SimTypeTop(SimType):
     """
@@ -144,6 +194,9 @@ class SimTypeTop(SimType):
     def c_repr(self):
         return "TOP"
 
+    def copy(self):
+        return SimTypeTop(size=self.size, label=self.label)
+
 
 class SimTypeReg(SimType):
     """
@@ -174,7 +227,10 @@ class SimTypeReg(SimType):
 
     def store(self, state, addr, value):
         store_endness = state.arch.memory_endness
-
+        try:
+            value = value.ast
+        except AttributeError:
+            pass
         if isinstance(value, claripy.ast.Bits):  # pylint:disable=isinstance-second-argument-not-valid-type
             if value.size() != self.size:
                 raise ValueError("size of expression is wrong size for type")
@@ -190,6 +246,9 @@ class SimTypeReg(SimType):
     def c_repr(self):
         return "<Reg_%d>" % self.size
 
+    def copy(self):
+        return self.__class__(self.size, label=self.label)
+
 
 class SimTypeNum(SimType):
     """
@@ -238,6 +297,9 @@ class SimTypeNum(SimType):
 
         state.memory.store(addr, value, endness=store_endness)
 
+    def copy(self):
+        return SimTypeNum(self.size, signed=self.signed, label=self.label)
+
 
 class SimTypeInt(SimTypeReg):
     """
@@ -310,6 +372,9 @@ class SimTypeInt(SimTypeReg):
             raise KeyError(k)
         return view._deeper(ty=ty)
 
+    def copy(self):
+        return self.__class__(signed=self.signed, label=self.label)
+
 
 class SimTypeShort(SimTypeInt):
     _base_name = 'short'
@@ -370,6 +435,9 @@ class SimTypeChar(SimTypeReg):
             ('label="%s"' % self.label) if self.label is not None else "",
         )
 
+    def copy(self):
+        return self.__class__(signed=self.signed, label=self.label)
+
 
 class SimTypeBool(SimTypeChar):
     def __repr__(self):
@@ -412,6 +480,9 @@ class SimTypeFd(SimTypeReg):
     def c_repr(self):
         return "fd_t"
 
+    def copy(self):
+        return SimTypeFd(label=self.label)
+
 
 class SimTypePointer(SimTypeReg):
     """
@@ -460,6 +531,50 @@ class SimTypePointer(SimTypeReg):
             self.offset
         )
 
+    def copy(self):
+        return SimTypePointer(self.pts_to, label=self.label, offset=self.offset)
+
+
+class SimTypeReference(SimTypeReg):
+    """
+    SimTypeReference is a type that specifies a reference to some other type.
+    """
+    def __init__(self, refs, label=None):
+        super().__init__(None, label=label)
+        self.refs: SimType = refs
+
+    def __repr__(self):
+        return "{}&".format(self.refs)
+
+    def c_repr(self):
+        return "{}&".format(self.refs.c_repr())
+
+    def make(self, refs):
+        new = type(self)(refs)
+        new._arch = self._arch
+        return new
+
+    @property
+    def size(self):
+        if self._arch is None:
+            raise ValueError("Can't tell my size without an arch!")
+        return self._arch.bits
+
+    def _with_arch(self, arch):
+        out = SimTypeReference(self.refs.with_arch(arch), label=self.label)
+        out._arch = arch
+        return out
+
+    def _init_str(self):
+        return "%s(%s%s)" % (
+            self.__class__.__name__,
+            self.refs._init_str(),
+            (', label="%s"' % self.label) if self.label is not None else "",
+        )
+
+    def copy(self):
+        return SimTypeReference(self.refs, label=self.label)
+
 
 class SimTypeFixedSizeArray(SimType):
     """
@@ -487,7 +602,7 @@ class SimTypeFixedSizeArray(SimType):
 
     def store(self, state, addr, values):
         for i, val in enumerate(values):
-            self.elem_type.store(state, addr + i*(self.elem_type.size//8), val)
+            self.elem_type.store(state, addr + i * (self.elem_type.size // state.arch.byte_width), val)
 
     @property
     def size(self):
@@ -509,6 +624,9 @@ class SimTypeFixedSizeArray(SimType):
             self.length,
         )
 
+    def copy(self):
+        return SimTypeFixedSizeArray(self.elem_type, self.length)
+
 
 class SimTypeArray(SimType):
     """
@@ -548,8 +666,11 @@ class SimTypeArray(SimType):
         out._arch = arch
         return out
 
+    def copy(self):
+        return SimTypeArray(self.elem_type, length=self.length, label=self.label)
 
-class SimTypeString(SimTypeArray):
+
+class SimTypeString(NamedTypeMixin, SimTypeArray):
     """
     SimTypeString is a type that represents a C-style string,
     i.e. a NUL-terminated array of bytes.
@@ -557,12 +678,12 @@ class SimTypeString(SimTypeArray):
 
     _fields = SimTypeArray._fields + ('length',)
 
-    def __init__(self, length=None, label=None):
+    def __init__(self, length=None, label=None, name: Optional[str]=None):
         """
         :param label:   The type label.
         :param length:  An expression of the length of the string, if known.
         """
-        super(SimTypeString, self).__init__(SimTypeChar(), label=label, length=length)
+        super().__init__(SimTypeChar(), label=label, length=length, name=name)
 
     def __repr__(self):
         return 'string_t'
@@ -607,16 +728,19 @@ class SimTypeString(SimTypeArray):
     def _with_arch(self, arch):
         return self
 
+    def copy(self):
+        return SimTypeString(length=self.length, label=self.label, name=self.name)
 
-class SimTypeWString(SimTypeArray):
+
+class SimTypeWString(NamedTypeMixin, SimTypeArray):
     """
     A wide-character null-terminated string, where each character is 2 bytes.
     """
 
     _fields = SimTypeArray._fields + ('length',)
 
-    def __init__(self, length=None, label=None):
-        super(SimTypeWString, self).__init__(SimTypeNum(16, False), label=label, length=length)
+    def __init__(self, length=None, label=None, name: Optional[str]=None):
+        super().__init__(SimTypeNum(16, False), label=label, length=length, name=name)
 
     def __repr__(self):
         return 'wstring_t'
@@ -662,6 +786,9 @@ class SimTypeWString(SimTypeArray):
     def _with_arch(self, arch):
         return self
 
+    def copy(self):
+        return SimTypeWString(length=self.length, label=self.label, name=self.name)
+
 
 class SimTypeFunction(SimType):
     """
@@ -682,7 +809,7 @@ class SimTypeFunction(SimType):
         super(SimTypeFunction, self).__init__(label=label)
         self.args = args
         self.returnty: Optional[SimType] = returnty
-        self.arg_names = arg_names if arg_names else []
+        self.arg_names = arg_names if arg_names else ()
         self.variadic = variadic
 
     def __repr__(self):
@@ -723,6 +850,53 @@ class SimTypeFunction(SimType):
             ", variadic=True" if self.variadic else "",
         )
 
+    def copy(self):
+        return SimTypeFunction(self.args, self.returnty, label=self.label, arg_names=self.arg_names,
+                               variadic=self.variadic)
+
+
+class SimTypeCppFunction(SimTypeFunction):
+    """
+    SimTypeCppFunction is a type that specifies an actual C++-style function with information about arguments, return
+    value, and more C++-specific properties.
+
+    :ivar ctor: Whether the function is a constructor or not.
+    :ivar dtor: Whether the function is a destructor or not.
+    """
+    def __init__(self, args, returnty, label=None, arg_names: Tuple[str]=None, ctor: bool=False, dtor: bool=False):
+        super().__init__(args, returnty, label=label, arg_names=arg_names, variadic=False)
+        self.ctor = ctor
+        self.dtor = dtor
+
+    def __repr__(self):
+        argstrs = [str(a) for a in self.args]
+        if self.variadic:
+            argstrs.append('...')
+        return '({}) -> {}'.format(', '.join(argstrs), self.returnty)
+
+    def c_repr(self):
+        return '({}) -> {}'.format(', '.join(str(a) for a in self.args), self.returnty)
+
+    def _init_str(self):
+        return "%s([%s], %s%s%s%s)" % (
+            self.__class__.__name__,
+            ", ".join([arg._init_str() for arg in self.args]),
+            self.returnty,
+            (", label=%s" % self.label) if self.label else "",
+            (", arg_names=[%s]" % self._arg_names_str(show_variadic=False)) if self.arg_names else "",
+            ", variadic=True" if self.variadic else "",
+        )
+
+    def copy(self):
+        return SimTypeCppFunction(
+            self.args,
+            self.returnty,
+            label=self.label,
+            arg_names=self.arg_names,
+            ctor=self.ctor,
+            dtor=self.dtor,
+        )
+
 
 class SimTypeLength(SimTypeLong):
     """
@@ -762,6 +936,9 @@ class SimTypeLength(SimTypeLong):
             self.size
         )
 
+    def copy(self):
+        return SimTypeLength(signed=self.signed, addr=self.addr, length=self.length, label=self.label)
+
 
 class SimTypeFloat(SimTypeReg):
     """
@@ -796,6 +973,9 @@ class SimTypeFloat(SimTypeReg):
     def c_repr(self):
         return 'float'
 
+    def copy(self):
+        return SimTypeFloat(self.size)
+
 
 class SimTypeDouble(SimTypeFloat):
     """
@@ -823,24 +1003,22 @@ class SimTypeDouble(SimTypeFloat):
             self.align_double
         )
 
+    def copy(self):
+        return SimTypeDouble(align_double=self.align_double)
 
-class SimStruct(SimType):
+
+class SimStruct(NamedTypeMixin, SimType):
     _fields = ('name', 'fields')
 
-    def __init__(self, fields, name=None, pack=False, align=None):
-        super(SimStruct, self).__init__(None)
+    def __init__(self, fields: Optional[Dict[str,SimType]], name=None, pack=False, align=None):
+        super().__init__(None, name='<anon>' if name is None else name)
         self._pack = pack
-        self._name = '<anon>' if name is None else name
         self._align = align
         self._pack = pack
         self.fields = fields
 
         self._arch_memo = {}
 
-    @property
-    def name(self): # required bc it's a property in the original
-        return self._name
-
     @property
     def offsets(self):
         offsets = {}
@@ -892,7 +1070,12 @@ class SimStruct(SimType):
 
     @property
     def size(self):
-        return sum(val.size for val in self.fields.values())
+        if not self.offsets:
+            return 0
+
+        last_name, last_off = list(self.offsets.items())[-1]
+        last_type = self.fields[last_name]
+        return last_off * self._arch.byte_width + last_type.size
 
     @property
     def alignment(self):
@@ -936,6 +1119,9 @@ class SimStruct(SimType):
             self._align,
         )
 
+    def copy(self):
+        return SimStruct(dict(self.fields), name=self.name, pack=self._pack, align=self._align)
+
 
 class SimStructValue:
     """
@@ -949,35 +1135,54 @@ class SimStructValue:
         self._struct = struct
         self._values = defaultdict(lambda: None, values or ())
 
+    def __indented_repr__(self, indent=0):
+        fields = []
+        for name in self._struct.fields:
+            value = self._values[name]
+            try:
+                f = getattr(value, '__indented_repr__')
+                s = f(indent=indent+2)
+            except AttributeError:
+                s = repr(value)
+            fields.append(' ' * (indent + 2) + '.{} = {}'.format(name, s))
+
+        return '{{\n{}\n{}}}'.format(',\n'.join(fields), ' ' * indent)
+
     def __repr__(self):
-        fields = ('.{} = {}'.format(name, self._values[name]) for name in self._struct.fields)
-        return '{{\n  {}\n}}'.format(',\n  '.join(fields))
+        return self.__indented_repr__()
 
     def __getattr__(self, k):
         return self[k]
 
     def __getitem__(self, k):
         if type(k) is int:
-            return self._values[self._struct.fields[k]]
+            k = self._struct.fields[k]
+        if k not in self._values:
+            for f in self._struct.fields:
+                if isinstance(f, NamedTypeMixin) and f.name is None:
+                    try:
+                        return f[k]
+                    except:
+                        continue
+            else:
+                return self._values[k]
+
         return self._values[k]
 
+    def copy(self):
+        return SimStructValue(self._struct, values=defaultdict(lambda: None, self._values))
 
-class SimUnion(SimType):
-    _fields = ('members', 'name')
+class SimUnion(NamedTypeMixin, SimType):
+    fields = ('members', 'name')
 
     def __init__(self, members, name=None, label=None):
         """
         :param members:     The members of the union, as a mapping name -> type
         :param name:        The name of the union
         """
-        super(SimUnion, self).__init__(label)
-        self._name = name if name is not None else '<anon>'
+        super().__init__(label, name=name if name is not None else '<anon>')
         self.members = members
 
-    @property
-    def name(self):
-        return self._name
-
     @property
     def size(self):
         return max(ty.size for ty in self.members.values())
@@ -986,6 +1191,24 @@ class SimUnion(SimType):
     def alignment(self):
         return max(val.alignment for val in self.members.values())
 
+    def _refine_dir(self):
+        return list(self.members.keys())
+
+    def _refine(self, view, k):
+        ty = self.members[k]
+        return view._deeper(ty=ty, addr=view._addr)
+
+    def extract(self, state, addr, concrete=False):
+        values = {}
+        for name, ty in self.members.items():
+            v = SimMemView(ty=ty, addr=addr, state=state)
+            if concrete:
+                values[name] = v.concrete
+            else:
+                values[name] = v.resolved
+
+        return SimUnionValue(self, values=values)
+
     def __repr__(self):
         # use the str instead of repr of each member to avoid exceed recursion
         # depth when representing self-referential unions
@@ -1002,6 +1225,59 @@ class SimUnion(SimType):
         out._arch = arch
         return out
 
+    def copy(self):
+        return SimUnion(dict(self.members), name=self.name, label=self.label)
+
+class SimUnionValue:
+    """
+    A SimStruct type paired with some real values
+    """
+    def __init__(self, union, values=None):
+        """
+        :param union:      A SimUnion instance describing the type of this union
+        :param values:      A mapping from union members to values
+        """
+        self._union = union
+        self._values = defaultdict(lambda: None, values or ())
+
+    def __indented_repr__(self, indent=0):
+        fields = []
+        for name, value in self._values.items():
+            try:
+                f = getattr(value, '__indented_repr__')
+                s = f(indent=indent+2)
+            except AttributeError:
+                s = repr(value)
+            fields.append(' ' * (indent + 2) + '.{} = {}'.format(name, s))
+
+        return '{{\n{}\n{}}}'.format(',\n'.join(fields), ' ' * indent)
+
+    def __repr__(self):
+        return self.__indented_repr__()
+
+    def __getattr__(self, k):
+        return self[k]
+
+    def __getitem__(self, k):
+        if k not in self._values:
+            return super().__getitem__(k)
+        return self._values[k]
+
+    def copy(self):
+        return SimUnionValue(self._union, values=self._values)
+
+
+class SimCppClass(SimStruct):
+    def __init__(self, members: Dict[str,SimType], name: Optional[str]=None, pack: bool=False, align=None):
+        super().__init__(members, name=name, pack=pack, align=align)
+
+    @property
+    def members(self):
+        return self.fields
+
+    def copy(self):
+        return SimCppClass(dict(self.fields), name=self.name, pack=self._pack, align=self._align)
+
 
 BASIC_TYPES = {
     'char': SimTypeChar(),
@@ -1065,7 +1341,11 @@ ALL_TYPES = {
     'string': SimTypeString(),
     'wstring': SimTypeWString(),
 
-    'va_list': SimStruct({}, name='va_list')
+    'va_list': SimStruct({}, name='va_list'),
+
+    # C++-specific
+    'basic_string': SimTypeString(),
+    'CharT': SimTypeChar(),
 }
 
 
@@ -1231,6 +1511,16 @@ def parse_file(defn, preprocess=True):
             ty = _decl_to_type(piece.type, extra_types)
             if piece.name is not None:
                 out[piece.name] = ty
+
+            # Don't forget to update typedef types
+            if (isinstance(ty, SimStruct) or isinstance(ty, SimUnion)) and ty.name != '<anon>':
+                for _, i in extra_types.items():
+                    if i.name == ty.name:
+                        if isinstance(ty, SimStruct):
+                            i.fields = ty.fields
+                        else:
+                            i.members = ty.members
+
         elif isinstance(piece, pycparser.c_ast.Typedef):
             extra_types[piece.name] = copy.copy(_decl_to_type(piece.type, extra_types))
             extra_types[piece.name].label = piece.name
@@ -1312,6 +1602,9 @@ def _decl_to_type(decl, extra_types=None):
 
     elif isinstance(decl, pycparser.c_ast.ArrayDecl):
         elem_type = _decl_to_type(decl.type, extra_types)
+
+        if decl.dim is None:
+            return SimTypeArray(elem_type)
         try:
             size = _parse_const(decl.dim)
         except ValueError as e:
@@ -1325,6 +1618,10 @@ def _decl_to_type(decl, extra_types=None):
         else:
             fields = OrderedDict()
 
+        # Don't forget that "type[]" has a different meaning in structures than in functions
+        if len(fields) > 0 and isinstance(fields[next(reversed(fields))], SimTypeArray):
+            raise NotImplementedError("Sorry, we have no support of flexible array members")
+
         if decl.name is not None:
             key = 'struct ' + decl.name
             struct = extra_types.get(key, None)
@@ -1379,12 +1676,16 @@ def _decl_to_type(decl, extra_types=None):
         else:
             raise TypeError("Unknown type '%s'" % ' '.join(key))
 
+    elif isinstance(decl, pycparser.c_ast.Enum):
+        # See C99 at 6.7.2.2
+        return ALL_TYPES['int']
+
     raise ValueError("Unknown type!")
 
 
 def _parse_const(c):
     if type(c) is pycparser.c_ast.Constant:
-        return int(c.value)
+        return int(c.value, base=0)
     elif type(c) is pycparser.c_ast.BinaryOp:
         if c.op == '+':
             return _parse_const(c.children()[0][1]) + _parse_const(c.children()[1][1])
@@ -1394,10 +1695,153 @@ def _parse_const(c):
             return _parse_const(c.children()[0][1]) * _parse_const(c.children()[1][1])
         if c.op == '/':
             return _parse_const(c.children()[0][1]) // _parse_const(c.children()[1][1])
+        if c.op == '<<':
+            return _parse_const(c.children()[0][1]) << _parse_const(c.children()[1][1])
+        if c.op == '>>':
+            return _parse_const(c.children()[0][1]) >> _parse_const(c.children()[1][1])
         raise ValueError('Binary op %s' % c.op)
     else:
         raise ValueError(c)
 
+
+def _cpp_decl_to_type(decl: Any, extra_types: Dict[str,SimType], opaque_classes=True):
+    if isinstance(decl, CppHeaderParser.CppMethod):
+        the_func = decl
+        func_name = the_func['name']
+        if "__deleting_dtor__" in func_name:
+            the_func['destructor'] = True
+        elif "__base_dtor__" in func_name:
+            the_func['destructor'] = True
+        elif "__dtor__" in func_name:
+            the_func['destructor'] = True
+        # translate parameters
+        args = [ ]
+        arg_names: List[str] = [ ]
+        for param in the_func['parameters']:
+            arg_type = param['type']
+            args.append(_cpp_decl_to_type(arg_type, extra_types, opaque_classes=opaque_classes))
+            arg_name = param['name']
+            arg_names.append(arg_name)
+
+        args = tuple(args)
+        arg_names: Tuple[str] = tuple(arg_names)
+        # returns
+        if not the_func['returns'].strip():
+            returnty = SimTypeBottom()
+        else:
+            returnty = _cpp_decl_to_type(the_func['returns'].strip(), extra_types, opaque_classes=opaque_classes)
+        # other properties
+        ctor = the_func['constructor']
+        dtor = the_func['destructor']
+        func = SimTypeCppFunction(args, returnty, arg_names=arg_names, ctor=ctor, dtor=dtor)
+        return func
+
+    elif isinstance(decl, str):
+        # a string that represents type
+        if decl.endswith("&"):
+            # reference
+            subdecl = decl.rstrip("&").strip()
+            subt = _cpp_decl_to_type(subdecl, extra_types, opaque_classes=opaque_classes)
+            t = SimTypeReference(subt)
+            return t
+
+        if decl.endswith(" const"):
+            # drop const
+            return _cpp_decl_to_type(decl[:-6].strip(), extra_types, opaque_classes=opaque_classes)
+
+        if "::" in decl:
+            unqualified_name = decl.split("::")[-1]
+        else:
+            unqualified_name = decl
+
+        key = unqualified_name
+        if key in extra_types:
+            t = extra_types[key]
+        elif key in ALL_TYPES:
+            t = ALL_TYPES[key]
+        elif opaque_classes is True:
+            # create a class without knowing the internal members
+            t = SimCppClass({}, name=decl)
+        else:
+            raise TypeError("Unknown type '%s'" % ' '.join(key))
+
+        if unqualified_name != decl:
+            t = t.copy()
+            t.name = decl
+        return t
+
+    raise NotImplementedError()
+
+
+def normalize_cpp_function_name(name: str) -> str:
+    _s = name
+    s = None
+    while s != _s:
+        _s = s if s is not None else _s
+        s = re.sub(r"<[^<>]+>", "", _s)
+
+    m = re.search(r"{([a-z\s]+)}", s)
+    if m is not None:
+        s = s[:m.start()] + "__" + m.group(1).replace(" ", "_") + "__" + s[m.end():]
+    return s
+
+
+def parse_cpp_file(cpp_decl, with_param_names: bool=False):
+    #
+    # A series of hacks to make CppHeaderParser happy with whatever C++ function prototypes we feed in
+    #
+
+    if CppHeaderParser is None:
+        raise ImportError("Please install CppHeaderParser to parse C++ definitions")
+
+    # CppHeaderParser does not support specialization
+    s = normalize_cpp_function_name(cpp_decl)
+
+    # CppHeaderParser does not like missing parameter names
+    # FIXME: The following logic is only dealing with *one* C++ function declaration. Support multiple declarations
+    # FIXME: when needed in the future.
+    if not with_param_names:
+        last_pos = 0
+        i = 0
+        while True:
+            idx = s.find(",", last_pos)
+            if idx == -1:
+                break
+            arg_name = "a%d" % i
+            i += 1
+            s = s[:idx] + " " + arg_name + s[idx:]
+            last_pos = idx + len(arg_name) + 1 + 1
+
+        # the last parameter
+        idx = s.find(")", last_pos)
+        if idx != -1:
+            # TODO: consider the case where there are one or multiple spaces between ( and )
+            if s[idx - 1] != "(":
+                arg_name = "a%d" % i
+                s = s[:idx] + " " + arg_name + s[idx:]
+
+    # CppHeaderParser does not like missing function body
+    s += "\n\n{}"
+
+    h = CppHeaderParser.CppHeader(s, argType="string")
+    if not h.functions:
+        return None, None
+
+    func_decls: Dict[str,SimTypeCppFunction] = { }
+    for the_func in h.functions:
+        # FIXME: We always assume that there is a "this" pointer but it is not the case for static methods.
+        proto: Optional[SimTypeCppFunction] = _cpp_decl_to_type(the_func, {}, opaque_classes=True)
+        if proto is not None and the_func['class']:
+            func_name = the_func['class'] + "::" + the_func['name']
+            proto.args = (SimTypePointer(pts_to=SimTypeBottom(label="void")),) + proto.args  # pylint:disable=attribute-defined-outside-init
+            proto.arg_names = ("this",) + proto.arg_names  # pylint:disable=attribute-defined-outside-init
+        else:
+            func_name = the_func['name']
+        func_decls[func_name] = proto
+
+    return func_decls, { }
+
+
 if pycparser is not None:
     _accepts_scope_stack()
 
